unit Engine;

interface

uses Windows, SysUtils, Classes, DXdraws, DxSounds, VFunc, VGraph, VConsts,
  StdObj, ExtObj, ContNRS, M3Consts, JPEG, GIFIMAGE, CtrlMsg,Graphics



  ;

type
  TMorkinEngine = class;

  TEngine = TMorkinEngine;



  TEngObj = class //(TComponent)
  private

    FX,
    FY: long;

  protected
    procedure SetX(Value: long); virtual;
    procedure SetY(Value: long); virtual;

    function GetImages: TTextureLib;
    function GetDrawer: TVSurfDrawer;

    function GetInView: boolean; virtual;
  public
    Engine: TEngine;

    constructor Create(AOwner: TComponent); overload; virtual; //override;
    constructor Create(AOwner: TComponent; Pos: TPoint); overload; virtual;
    destructor Destroy; override;

    property Drawer: TVSurfDrawer read GetDrawer;
    property Images: TTextureLib read GetImages;

    property InView: boolean read GetInView;
  published
    property X: long read FX write SetX;
    property Y: long read FY write SetY;

  end;

  TNonVis = class(TEngObj)
  end;


  TGraphObj = class(TEngObj)
  private
    FImage: string;
  public

    constructor Create(AOwner: TComponent; Pos: TPoint; Img: string); overload; virtual;

    procedure Paint; virtual; abstract;
  published
    property Image: string read FImage write FImage;
  end;

  TSpellFX = class(TGraphObj)
  end;

  TTerrain = class(TGraphObj)
  private

  protected

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Paint; override;
  published

  end;

  TMapObject = class(TGraphObj)
  end;

  TActivator=class(TMapObject)
  end;


  TPlayer = class;

  TChar = class(TGraphObj)
  private
    FHP,
    FMana,
    FMP:long;

    FMaxHP,
    FMaxMana,
    FMaxMP:long;


  protected
     procedure SetHP(Value:long);virtual;
     procedure SetMana(Value:long);virtual;
     procedure SetMP(Value:long);virtual;
     procedure SetMaxHP(Value:long);virtual;
     procedure SetMaxMana(Value:long);virtual;
     procedure SetMaxMP(Value:long);virtual;

  public
    Player: TPlayer;

    constructor Create(AOwner:TComponent;Pos:TPoint;Img:string;Pl:TPlayer);overload;virtual;
    constructor Create(AOwner:TComponent;Pos:TPoint;Img:string);overload;override;

            //  destructor Destroy; override;
    procedure Paint; override;
    procedure Input(MSG: TVMSG); virtual;

    property HP:long read FHP write SetHP;
    property Mana:long read FMana write SetMana;
    property MP:long read FMP write SetMP;
    property MaxHP:long read FMaxHP write SetMaxHP;
    property MaxMana:long read FMaxMana write SetMaxMana;
    property MaxMP:long read FMaxMP write SetMaxMP;



  end;

  TPlayer = class(TChar)
  private

  protected
   function GetUnits:long;

  public
    Color:TColor;

    constructor Create(AOwner:TComponent;Pos:TPoint;Img:string);override;

    procedure Input(MSG: TVMSG); override;

    property Units:long read GetUnits;

    // procedure Paint;override;
  end;



///////////////////////////////

  TTerrainLayer = array[0..MAX_MAPSIZE - 1, 0..MAX_MAPSIZE - 1] of TTerrain;

  TMorkinEngine = class(TComponent)
  private
    FProtChars: TObjectList;
    FProtTerrain: TObjectList;
    FProtSpells: TObjectList;
    FProtObjects: TObjectList;
    FProtNonVis: TObjectList;
    FProtPlayers: TObjectList;

    FTerrain: TObjectList;
    FChars: TObjectList;
    FSpells: TObjectList;
    FObjects: TObjectList;
    FPlayers: TObjectList;
    FNonVis: TObjectList;


    FLayer: TTerrainLayer;


  protected
    function GetTerrain(index: long): TTerrain;
             {  function GetTerrain(index:long):TTerrain;
               function GetTerrain(index:long):TTerrain;
               function GetTerrain(index:long):TTerrain;
               function GetTerrain(index:long):TTerrain;
              }
    function GetViewSize: TPoint;
    procedure DoDraw(Sender: TObject);

    procedure FreeLayer;
  public
    Images: TTextureLib;
    Drawer: TVSurfDrawer;
    TopLeft: TPoint;
    TopLeftPix: TPoint;
    MapSize: TPoint;

    Draw: TCustomDXDraw;

    All: TObjectList;

    APlayer: TPlayer;
    AUnit: TChar;

    Inited:boolean;
    Loaded:boolean;

    constructor Create(AOWner: TComponent); override;
    destructor Destroy; override;

    procedure Paint;

    procedure Load;
    procedure UnLoad;
    procedure Center(P:TPoint);
    procedure Input(MSG: TVMSG);

    procedure Init;

    procedure NextUnit(Cen:boolean=true);
    procedure NextPlayer(Cen:boolean=true);
    procedure PrevUnit(Cen:boolean=true);
    procedure PrevPlayer(Cen:boolean=true);

    function Add(Obj: TEngObj): TEngObj;

    procedure RandomMap;

    property Terrain[index: long]: TTerrain read GetTerrain;

    property ViewSize: TPoint read GetViewSize;
  published

  end;












implementation

uses GameClass, MorkinClass;
//////////////  TENGOBJ METHODS /////////////////////////////

procedure TEngObj.SetX(Value: long);
begin
  FX := Value;

end;

procedure TEngObj.SetY(Value: long);
begin
  FY := Value;
end;

function TEngObj.GetImages: TTextureLib;
begin
  Result := Engine.Images;
end;

function TEngObj.GetDrawer: TVSurfDrawer;
begin
  Result := Engine.Drawer;
end;

function TEngObj.GetInView: boolean;
begin
  Result := ((X >= Engine.TopLeft.x) and (X <= Engine.TopLeft.x + Engine.ViewSize.X))
    and ((X >= Engine.TopLeft.x) and (X <= Engine.TopLeft.x + Engine.ViewSize.X));

end;

constructor TEngObj.Create(AOwner: TComponent);
begin
  inherited Create; //(AOwner);
  Engine := TEngine(AOwner);
  FX := 0;
  FY := 0;
end;

constructor TEngObj.Create(AOwner: TComponent; Pos: TPoint);
begin
  Create(AOwner);
  X := Pos.X;
  Y := Pos.Y;
end;


destructor TEngObj.Destroy;
begin

  inherited Destroy;
end;

//////////////  TGraphObj METHODS       ///////////////

constructor TGraphObj.Create(AOwner: TComponent; Pos: TPoint; Img: string);
begin
  Create(AOwner, Pos);
  Image := Img;

end;
//////////////  TTERRAIN METHODS       ///////////////

constructor TTerrain.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Image := 'Grass';

end;



procedure TTerrain.Paint;
begin
  if not InView then Exit;
  if Images.Items.Find(Image) <> nil then
    Images.Items.Find(Image).Draw(Drawer.Surface.Surf, (X - Engine.TopLeft.X) * Tile_Size, (Y - Engine.TopLeft.Y) * Tile_Size, 0);

end;


destructor TTerrain.Destroy;
begin

  inherited Destroy;
end;


//////////////////////////////////////
constructor TChar.Create(AOwner:TComponent;Pos:TPoint;Img:string;Pl:TPlayer);
 begin
  Create(AOwner,Pos,Img);
  Player:=Pl;

 end;

constructor TChar.Create(AOwner:TComponent;Pos:TPoint;Img:string);
 begin
  inherited Create(AOwner,Pos,Img);
  FMaxHP:=20;
  FMaxMana:=15;
  FMaxMP:=5;
  FHP:=20;
  FMana:=15;
  FMP:=5;
 end;

procedure TChar.SetHP(Value:long);
begin
 FHP:=Value;
end;

procedure TChar.SetMana(Value:long);
begin
 FMana:=Value;

end;

procedure TChar.SetMP(Value:long);
begin
 FMP:=Value;

end;

procedure TChar.SetMaxHP(Value:long);
begin
 FMaxHP:=Value;

end;

procedure TChar.SetMaxMana(Value:long);
begin
 FMaxMana:=Value;

end;

procedure TChar.SetMaxMP(Value:long);
begin
 FMaxMP:=Value;

end;




procedure TChar.Paint;
begin
  if not InView then Exit;
  if Images.Items.Find(Image) <> nil then
    Images.Items.Find(Image).Draw(Drawer.Surface.Surf, (X - Engine.TopLeft.X) * Tile_Size, (Y - Engine.TopLeft.Y) * Tile_Size, 0);

end;

procedure TChar.Input(MSG: TVMSG);
begin
  case MSG.Message of
    MSG_KEYDOWN:
      begin
      case MSG.Key of
        VK_UP: Dec(FY);
        VK_DOWN: Inc(FY);
        VK_RIGHT: Inc(FX);
        VK_LEFT: Dec(FX);
      end;
       Engine.Center(Point(X,Y));

      end;

  end;
end;


///////////////////////////////////////
constructor TPlayer.Create(AOwner:TComponent;Pos:TPoint;Img:string);
 begin
  inherited Create(AOwner,Pos,Img);
  Player:=Self;
 end;

procedure TPlayer.Input(MSG: TVMSG);
begin
  inherited;
  {case MSG.Key of
   VK_UP:Dec(FY);
   VK_DOWN:Inc(FY);
   VK_RIGHT:Inc(FX);
   VK_LEFT:Dec(FX);
  end;  }
end;

function TPlayer.GetUnits:long;
var i:long;
 begin
  Result:=0;
  for i:=0 to Engine.FChars.Count-1 do
   if TChar(Engine.FChars[i]).Player=Self then Inc(Result);

 end;


/////////////////////////////////////////////////////////
////////////////  TMORKIN ENGINE METHODS //////////////
//////////////////////////////////////////////////////

constructor TEngine.Create(AOWner: TComponent);
begin
  inherited Create(AOwner);
  Draw := TVGame(AOwner).Draw;
  Images := TTextureLib.Create(AOwner);
  Images.DXDraw := Draw;
  Drawer := TMorkin3(AOwner).Wins.SurfDrawer;
  Drawer.OnDraw := DoDraw;
  All := TObjectList.Create;
  FTerrain := TObjectList.Create;
  FChars := TObjectList.Create;
  FSpells := TObjectList.Create;
  FObjects := TObjectList.Create;
  FPlayers := TObjectList.Create;
  FNonVis := TObjectList.Create;

  MapSize.X := 100;
  MapSize.Y := 100;


end;

destructor TEngine.Destroy;
begin
  Images.Free;
  inherited Destroy;
end;

procedure TEngine.RandomMap;
var
  u, i, ii: long;

begin


//  FreeLayer;

  for i := 0 to MAPSIZE.X - 1 do
    for ii := 0 to MAPSIZE.Y - 1 do
    begin
      u := random(4);

      case u of

      0,3:  Add(TTerrain.Create(Self, Point(i, ii), 'GRASS'));
       1: Add(TTerrain.Create(Self, Point(i, ii), 'DIRT'))  ;
        //3:  Add(TTerrain.Create(Self, Point(i, ii), 'SNOW')) ;
       2: Add(TTerrain.Create(Self, Point(i, ii), 'ROCKYSAND'));


      end;

    end;

  Add(TPlayer.Create(Self, Point(5, 5), 'Red Mage'));


  APlayer := TPlayer(FPlayers[0]);

  Add(TChar.Create(Self,Point(0,1),'Necromancer',APlayer));
  Add(TChar.Create(Self,Point(3,1),'Druid',APlayer));
  Add(TChar.Create(Self,Point(5,9),'Necromancer',APlayer));
  Add(TChar.Create(Self,Point(10,3),'Druid',APlayer));
  Add(TChar.Create(Self,Point(7,7),'Necromancer',APlayer));

  Add(TPlayer.Create(Self, Point(5, 5), 'Blue Mage'));
  APlayer := TPlayer(FPlayers[1]);

  Add(TChar.Create(Self,Point(5,1),'Knight',APlayer));
  Add(TChar.Create(Self,Point(9,1),'Knight',APlayer));
  Add(TChar.Create(Self,Point(5,2),'Elf',APlayer));
  Add(TChar.Create(Self,Point(10,7),'Elf',APlayer));
  Add(TChar.Create(Self,Point(4,9),'Knight',APlayer));

  APlayer := TPlayer(FPlayers[0]);


  NextUnit(false);

end;

procedure TEngine.Input(MSG: TVMSG);
var
  P: TPoint;
begin
  P := Point(MSG.POS.X - Drawer.GetPRect.Left, MSG.POS.Y - Drawer.GetPRect.Top);

  case MSG.Message of
    MSG_MOUSEDOWN:
      if Drawer.Mousein2 and Drawer.GetMouseRect then
      begin
        Center(Point(TopLeft.X + (P.X div TILE_SIZE),TopLeft.Y + (P.Y div TILE_SIZE)));
      end;
  end;
  if AUnit <> nil then AUnit.Input(MSG);

end;

procedure TEngine.Center(P:TPoint);
 begin

  Topleft := Point(-ViewSIZE.X div 2 + P.X, -ViewSIZE.X div 2 + P.Y);
  TopLeft.X := Max(0, Min(Topleft.X, MapSize.X - ViewSize.X));
  TopLeft.Y := Max(0, Min(Topleft.Y, MapSize.Y - ViewSize.Y));

 end;

procedure TEngine.Load;
var
  Temp: TDXImageList;
begin
 { FTerrain.Clear;
  FPlayers.Clear;
  FChars.Clear;
  FSpells.Clear;
  FObjects.Clear;
  All.Clear;
  }
  Temp := TDXImageList.Create(Self);
  Temp.Items.LoadFromFile(ExeDir + FILE_GFX_TERRAIN);
  Images := AddIList(Images, Temp);
  Temp.Items.LoadFromFile(ExeDir + FILE_GFX_OBJECTS);
  AddIList(Images, Temp);
  Temp.Items.LoadFromFile(ExeDir + FILE_GFX_CHARS);
  AddIList(Images, Temp);
  Temp.Items.LoadFromFile(ExeDir + FILE_GFX_SPELLS);
  AddIList(Images, Temp);
  Temp.Free;

  RandomMap;

  Loaded:=true;
  Inited:=false;


  end;

procedure TEngine.UnLoad;
var
  x, y: long;
begin
  Images.Items.Clear;
  FreeLayer;

  FChars.Clear;
  FSpells.Clear;
  FObjects.Clear;
  All.OwnsObjects:=false;
  All.Clear;
  All.OwnsObjects:=true;
  FPlayers.OwnsObjects:=false;
  FPlayers.Clear;
  FPlayers.OwnsObjects:=true;

  Loaded:=false;
  Inited:=false;

end;

procedure TEngine.DoDraw(Sender: TObject);
begin
  Paint;
end;

procedure TEngine.Paint;
var
  i, x, y: long;
begin
 // Drawer.Surface.Surf.Fill(200);
 if not Loaded then Exit;

  for x := TopLeft.X to TopLeft.X + ViewSize.X + 1 do
    for y := TopLeft.Y to TopLeft.Y + ViewSize.Y + 1 do
      if FLayer[x, y] <> nil then FLayer[x, y].Paint;

  for i := 0 to FChars.Count - 1 do
    TChar(FChars[i]).Paint;

  if APlayer<>nil then APlayer.Paint;
  if AUnit<>nil then AUnit.Paint;

  if Loaded and (not Inited) then Init;  
  end;

function TEngine.GetViewSize: TPoint;
begin

  Result.X := Drawer.Surface.Surf.Width div TILE_SIZE;
  if (Drawer.Surface.Surf.Width mod TILE_SIZE) <> 0 then Inc(Result.X);

  Result.Y := Drawer.Surface.Surf.Height div TILE_SIZE;
  if (Drawer.Surface.Surf.Height mod TILE_SIZE) <> 0 then Inc(Result.Y);

end;

function TEngine.GetTerrain(index: long): TTerrain;
begin
  Result := TTerrain(FTerrain[index]);

end;



function TEngine.Add(Obj: TEngObj): TEngObj;
var
  i: long;
begin

  if Obj is TTerrain then
  begin
    FTerrain.Add(Obj);
    if FLayer[TTerrain(Obj).X, TTerrain(Obj).Y] <> nil then
    begin
      i := All.IndexOf(FLayer[TTerrain(Obj).X, TTerrain(Obj).Y]);
      FTerrain.Remove(FLayer[TTerrain(Obj).X, TTerrain(Obj).Y]);
      All.Delete(i);
    end;
    FLayer[TTerrain(Obj).X, TTerrain(Obj).Y] := TTerrain(Obj);
  end;

  if Obj is TChar then FChars.Add(Obj);
  if Obj is TMapObject then FObjects.Add(Obj);
  if Obj is TNonVis then FNonVis.Add(Obj);
  if Obj is TSpellFX then FSpells.Add(Obj);
  if Obj is TPlayer then FPlayers.Add(Obj);

  All.Add(Obj);


end;

procedure TEngine.FreeLayer;
var
  x, y: long;
begin
  for x := 0 to MAX_MAPSIZE - 1 do
    for y := 0 to MAX_MAPSIZE - 1 do
    begin
      if Flayer[x, y] <> nil then
      begin
        All.OwnsObjects := false;
        All.Remove(Flayer[x, y]);
        FTerrain.Remove(Flayer[x, y]);
        All.OwnsObjects := true;
        end;
      FLayer[x, y] := nil;

    end;

  FTerrain.OwnsObjects:=false;
  FTerrain.Clear;
  FTerrain.OwnsObjects:=true;


end;


procedure TEngine.NextUnit(Cen:boolean);
var i,idx:long;
begin
 if APlayer=nil then NextPlayer;
 if AUnit=nil then idx:=0
  else idx:=FChars.IndexOf(AUnit)+1;

 repeat
  if idx>FChars.Count-1 then idx:=0;
  AUnit:=TChar(FChars[idx]);
  Inc(idx);

 until AUnit.Player=APlayer;

 if Cen then Center(Point(AUnit.X,AUnit.Y));
 if Cen then TMorkin3(Owner).Updater.UpdateUnitAll(AUnit);
end;

procedure TEngine.NextPlayer(Cen:boolean);
var idx:long;
begin
 if APlayer=nil then idx:=0
  else idx:=FPlayers.IndexOf(APlayer)+1;
 if idx>FPlayers.Count-1 then idx:=0;

 APlayer:=TPlayer(FPlayers[idx]);
 AUnit:=nil;
 NextUnit(cen);
 if Cen then TMorkin3(Owner).Updater.UpdatePlayerAll(APlayer);

end;

procedure TEngine.PrevUnit(Cen:boolean);
var i,idx:long;
begin
 if APlayer=nil then PrevPlayer;
 if AUnit=nil then idx:=FChars.Count-1
  else idx:=FChars.IndexOf(AUnit)-1;

 repeat
  if idx<0 then idx:=FChars.Count-1;
  AUnit:=TChar(FChars[idx]);
  Dec(idx);

 until AUnit.Player=APlayer;
 if Cen then Center(Point(AUnit.X,AUnit.Y));
 if Cen then TMorkin3(Owner).Updater.UpdateUnitAll(AUnit);

 end;


procedure TEngine.PrevPlayer(Cen:boolean);
var idx:long;
begin
 if APlayer=nil then idx:=FPlayers.Count-1
  else idx:=FPlayers.IndexOf(APlayer)-1;

 if idx<0 then idx:=FPlayers.Count-1;

 APlayer:=TPlayer(FPlayers[idx]);
 AUnit:=nil;
 NextUnit(cen);
 if Cen then TMorkin3(Owner).Updater.UpdatePlayerAll(APlayer);

end;


procedure TEngine.Init;
begin
Inited:=
TMorkin3(Owner).Updater.UpdateUnitAll(AUnit)and
TMorkin3(Owner).Updater.UpdatePlayerAll(APlayer);

 
end;







end.

