unit GameClass;

interface

uses Windows, Classes, SysUtils, Controller, DXDraws, DXSounds, MIDPlay, MIDList,
  AStream, VStream, VTextList, Graphics, VFunc, VGraph, VConsts, Forms, Dxclass, CtrlMSG,
  StdObj,inifiles


  ;


type

  TVGame = class(TComponent)
  private
    FDraw: TCustomDXDraw;
    FSound:TCustomDXSound;
    FState: long;

    FFull:boolean;
    FSysmem:boolean;
    FVBlank:boolean;

  protected
    procedure SetDraw(Value: TCustomDxDraw); virtual;
    procedure SetState(Value: long); virtual;
    procedure SetFullScr(Value:boolean);
    procedure SetSysmem(Value:boolean);
    procedure SetVBlank(Value:boolean);
    procedure SetDispWidth(Value:long);virtual;
    procedure SetDispHeight(Value:long);virtual;
    procedure SetDispBits(Value:long);virtual;
    function GetDispWidth:long;virtual;
    function GetDispHeight:long;virtual;
    function GetDispBits:long;virtual;

    procedure NotifyEvent(Sender: TCustomDXDraw; Notify: TDXDrawNotifyType); virtual;

  public
    Title: string;
    Developer: string;
    WinFile: string;
    CfgFile:string;

    CanInit:boolean;

    INIFile:TINIFile;

    Timer: TDXTimer;
    WC: TController;
    MIDList: TMIDList;
    MIDPlay: TMIDPlay;
    VStream: TVStream;
    AStream: TAStream;
    TextList: TTextList;

    ShowFPS: boolean;
    PlayMIDI:boolean;


    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetComponents(AWC: TController;
      FML: TMIDList;
      FMP: TMIDPlay;
      FVS: TVStream;
      FAS: TAStream;
      FTL: TTextList);

    procedure LoadWindows(FileName: string); virtual;
    procedure LoadData;virtual;abstract;
    procedure ReadCfg;virtual;abstract;
    procedure WriteCfg;virtual;abstract;
    procedure SetDefaults;virtual;
    procedure Paint; virtual;
    procedure Define; virtual;
    procedure Quit; virtual;
    procedure SetRes(W,H,B:long);virtual;
    procedure BeforeDestruction;override;

    procedure CtrlHandler(Win: TVWinControl; Ctrl: TVControl; MSG: TVMSG); virtual;
    procedure GlobalHandler(MSG: TVMSG); virtual;

    procedure ShowWindows(Wins: array of TVForm;Hide:boolean=true); virtual;

    property Draw: TCustomDXDraw read FDraw write SetDraw;
    property Sound:TCustomDXSound read FSound write FSound;
    property State: long read FState write SetState;

    property FullScreen:boolean read FFull write SetFullScr;
    property SystemMem:boolean read FSysMem write SetSysMem;
    property VBlank:boolean read FVBlank write SetVBlank;

    property DisplayWidth:long read GetDispWidth write SetDispWidth;
    property DisplayHeight:long read GetDispHeight write SetDispHeight;
    property DisplayBitCount:long read GetDispBits write SetDispBits;

  published

  end;




implementation
uses MainForm;

constructor TVGame.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  CanInit:=false;
  Define;
  Draw := VFunc.SearchDXDraw(TForm(AOwner));
  Sound:=VFunc.SearchDXSound(TForm(AOwner));
  WC := TController.Create(AOwner);
  WC.Draw:=draw;
  WC.ControlEvent := CtrlHandler;
  WC.GlobalEvent := GlobalHandler;
  MIDList := TMIDList.Create(AOwner);
  MIDPlay := TMIDPlay.Create(AOwner);
  MIDPlay.Init;
  MIDPlay.PlayList.StoreList := MIDList;
  TextList:=TTextList.Create(AOwner);

  LoadData;
  LoadWindows(WinFile);

  SetDefaults;
  INIFile:=TINIFile.Create(CfgFile);
  ReadCfg;
  State := 0;
  CanInit:=true;
  Draw.Initialize;
end;

destructor TVGame.Destroy;
begin

  inherited Destroy;
end;

procedure TVGame.Quit;
begin
// TForm(Owner).Close;
// BeforeDestruction;
// ExitApp;
//BeforeDestruction;
//Free;
WriteCfg;
//INIFile.Free;


ExitApp;

end;

procedure TVGame.SetDraw(Value: TCustomDXDraw);
begin
  if FDraw <> Value then
  begin
    if FDraw <> nil then
      FDraw.UnRegisterNotifyEvent(NotifyEvent);
    FDraw := Value;
    if FDraw <> nil then
    begin
      FDraw.RegisterNotifyEvent(NotifyEvent);
      //FImages.DXDraw := FDraw;
      //FImages.Items.Restore;
    end;
  end;
end;

procedure TVGame.SetState(Value: long);
begin
  FState := Value;

end;

procedure TVGame.NotifyEvent(Sender: TCustomDXDraw; Notify: TDXDrawNotifyType);
begin


end;


procedure TVGame.Define;
begin
  Title := 'Unknown Game';
  Developer := 'Unknown';
  WinFile := 'nil';
  ShowFPS := false;

end;

procedure TVGame.SetComponents(AWC: TController;
  FML: TMIDList;
  FMP: TMIDPlay;
  FVS: TVStream;
  FAS: TAStream;
  FTL: TTextList);
begin





end;


procedure TVGame.Paint;
begin
  if WC <> nil then WC.Paint;

  if ShowFPS then
    with Draw.Surface.Canvas do
    begin

      SetBkMode(Handle, Transparent);
      Brush.Color := clBlack;
      Font.Size := 12;
      Font.Color := clRED;
      Font.Style := [fsBold];
      Font.name := 'Arial';
      TextOut(10, 10, 'FPS: ' + IntToStr(Timer.Framerate));
      SetBkMode(Handle, Opaque);
      TextOut(10, 40, 'Mem: ' + IntToStr(AllocmemSize));

      Release;

    end;



end;


procedure TVGame.LoadWindows(FileName: string);
begin
  if WC <> nil then WC.LoadAll(Filename);

end;




procedure TVGame.CtrlHandler(Win: TVWinControl; Ctrl: TVControl; MSG: TVMSG);
begin




end;

procedure TVGame.GlobalHandler(MSG: TVMSG);
begin




end;

procedure TVGame.ShowWindows(Wins: array of TVForm;Hide:boolean);
var
  i: long;
begin
  if Hide then WC.HideAll;
  for i := Low(Wins) to High(Wins) do
    if Wins[i]<>nil then Wins[i].Show;

end;


procedure TVGame.SetFullScr(Value:boolean);
 begin
  FFull:=Value;
  if FFull=true then
   begin
   TForm(Owner).BorderStyle:=bsNone;
   Draw.Options:=Draw.Options+[doFullScreen]
   end
   else
   begin
   TForm(Owner).BorderStyle:=bsSingle;
   Draw.Options:=Draw.Options-[doFullScreen];
   end;
  SetRes(DisplayWidth,DisplayHeight,DisplayBitCount);
  if CanInit then Draw.Initialize;

 end;

procedure TVGame.SetSysmem(Value:boolean);
 begin
  FSysMem:=Value;
  if FSysMem=true then
   Draw.Options:=Draw.Options+[doSystemMemory]
   else
   Draw.Options:=Draw.Options-[doSystemMemory];
  if CanInit then Draw.Initialize;

 end;


procedure TVGame.SetVBlank(Value:boolean);
 begin
  FVBlank:=Value;
  if FVBlank=true then
   Draw.Options:=Draw.Options+[doWaitVBlank]
   else
   Draw.Options:=Draw.Options-[doWaitVBlank];
  if CanInit then Draw.Initialize;

 end;

procedure TVGame.SetRes(W,H,B:long);
 begin
  if FullScreen then
  begin
  Draw.Display.Width:=W;
  Draw.Display.Height:=H;
  Draw.Display.BitCount:=B;
  end
  else
  begin
  TForm(Owner).ClientWidth:=W;
  TForm(Owner).ClientHeight:=H;
  end;
  if CanInit then Draw.Initialize;
 end;

procedure TVGame.SetDefaults;
 begin
  ShowFPS:=false;
  FFull:=true;
  FSysMem:=false;
  FVBlank:=true;
  PlayMIDI:=true;
 end;

procedure TVGame.SetDispWidth(Value:long);
begin
 SetRes(Value,DisplayHeight,DisplayBitCount);
end;

procedure TVGame.SetDispHeight(Value:long);
begin
 SetRes(DisplayWidth,Value,DisplayBitCount);
end;


procedure TVGame.SetDispBits(Value:long);
begin
 SetRes(DisplayWidth,DisplayHeight,Value);
end;


function TVGame.GetDispWidth:long;
begin
 Result:=Draw.SurfaceWidth;
end;


function TVGame.GetDispHeight:long;
begin
 Result:=Draw.SurfaceHeight;

end;


function TVGame.GetDispBits:long;
begin
 if FullScreen then Result:=Draw.Display.BitCount
 else Result:=GetDeviceCaps(GetDC(0),BITSPIXEL);

end;

procedure TVGame.BeforeDestruction;
 begin
  WriteCfg;
  //INIFile.Free;

 end;

end.

